var Logger = (function() 
{
	var context = {};
	
	var ready = false;
	var level = 0;
	var localDirectoryEntry = null;
	var cache = '';
	var cacheSize = 512;
	var limitFileSize = 5 * 1024 * 1024;
	var filename = 'log0.txt';
	var storedFilename = 'log1.txt';	
	
	this.init = function(_directory, _callback) 
	{					
		var success = function(dir)
		{
			Debug.Log('FileSystem loaded');           

			if (typeof _callback != 'undefined' && _callback !== null)
			{
				_callback(dir);
			}
		};

		var fail = function(error) 
		{
			Debug.Log('Error loading File System , code: ' + error.code);
		};

		window.resolveLocalFileSystemURL(_directory, success, fail);
	};
	
	this.copy = function(_to)
	{		
		var fail = function(error) 
		{
			Debug.Log('Error copy file, code: ' + error.code);
		}; 
		
		var success = function(dirEntry)
		{
			localDirectoryEntry.getFile(filename, {}, function(fileEntry) 
			{
				fileEntry.copyTo(dirEntry);
				
			}, fail);
			
			localDirectoryEntry.getFile(storedFilename, {}, function(fileEntry) 
			{
				fileEntry.copyTo(dirEntry);
				
			}, fail);
		};				
		
		init(cordova.file.externalApplicationStorageDirectory, function(_dir) 
		{
			_dir.getDirectory(_to, {}, success, fail);
		});					
	};
	
	this.remove = function(_dir, _filename)
	{
		var fail = function(error) 
		{
			Debug.Log('Error removing file, code: ' + error.code);
		}; 
		
		var success = function(fileEntry)
		{
			fileEntry.remove(function() 
			{
				Debug.Log('File removed.');
				
			}, fail);
		};				
		
		_dir.getFile(filename, {create: true}, success, fail);
	};	
	
	this.append = function(_dir, _data, _callback) 
	{
		if (!ready)
		{
			return;
		}
		
		var fail = function(error) 
		{
			Debug.Log('Error saving file, code: ' + error.code);
			if (error.code == 10) // QUOTA_EXCEED
			{
				copy('');
				setTimeout(function() 
				{
					remove(_dir, filename);
					remove(_dir, storedFilename);
				}, 2000);
			}
		}; 
		
		var success = function(fileEntry) 
		{
			fileEntry.createWriter(function(fileWriter) 
			{
				if (fileWriter.length < limitFileSize)
				{
					fileWriter.seek(fileWriter.length);
				}
				else
				{
					fileEntry.moveTo(_dir, storedFilename);
				}
				
				fileWriter.onwriteend = function(e) 
				{
					Debug.Log('Write completed.');
					if (typeof _callback != 'undefined' && _callback !== null)
					{
						_callback();
					}
				};

				fileWriter.onerror = function(e) 
				{
					Debug.Log('Write failed: ' + e.toString());
				};

				fileWriter.write(_data);

			}, fail);

		};				              	
		
		_dir.getFile(filename, {create: true}, success, fail);		
	};
	
	this.flush = function(_data, _callback)
	{
		if (!ready)
		{
			init(cordova.file.dataDirectory, function(_dir) 
			{
				ready = true;
				localDirectoryEntry = _dir;
				
				append(_dir, _data, _callback);				
			});
		}
		else
		{
			append(localDirectoryEntry, _data, _callback);
		}
	};
	
	this.save = function(_data, _force, _callback)
	{
		try
		{				
			_data = (new Date().getTime()) + ": " + _data;			
		    _data = enc(_data, 127);			    
			_data = pako.deflate(_data, { to: 'string' });			
			// Il primo byte indica la versione
			_data = "0" + btoa(_data) + "\r\n";		
			
			cache += _data;
			
			if (cache.length > cacheSize || _force)
			{
				flush(cache, _callback);
				cache = '';
			}			
		}
		catch (e)
		{
			Debug.Log('FileSystem exception [log]: ' + e);
		}
	};
	
	this.enc = function(str, secret) 
	{
	    var encoded = "";
	    for (var i = 0; i < str.length; ++i) 
	    {
	        var a = str.charCodeAt(i);
	        var b = a ^ secret; 
	        encoded = encoded + String.fromCharCode(b);
	    }
	    return encoded;
	};
	
	
	context.NO_LOG = 0;		
	context.NORMAL = 1;
	context.VERBOSE = 2;			
	
	context.log = function(_level, _data) 
	{
		if (_level < level)
		{
			save(_data, false);
		}
	};
	
	context.transfer = function()
	{			
		try
		{
			if (!ready)
			{
				save('Logger.transfer', true, function() 
				{
					copy('');
				});
			}
			else
			{
				copy('');
			}
		}
		catch (e)
		{
			Debug.Log('FileSystem exception [persist]: ' + e);
		}
	};		
	
	context.enable = function(_level)
	{
		level = _level + 1;
	};
	
	context.disable = function()
	{
		level = context.NO_LOG;
	};
	
	return context;
	
})();
